;; Generated package description from eev.el  -*- no-byte-compile: t -*-
(define-package "eev" "20191105" "Support for e-scripts (eepitch blocks, elisp hyperlinks, etc)" '((emacs "24.1")) :keywords '("lisp" "e-scripts") :authors '(("Eduardo Ochs" . "eduardoochs@gmail.com")) :maintainer '("Eduardo Ochs" . "eduardoochs@gmail.com") :url "http://angg.twu.net/#eev")
